/*	maze.h -- maze-solving
*
*	Written by Cathy Saxton
*	cathy@idleloop.com
*/

#ifndef MAZE_H
#define MAZE_H

#ifndef WIN
#include "eeprom.h"
#endif

/* turn commands (returned from TurnAtJct) */
enum	// note: order assumed!!! (clockwise)
{
	turnNone = 0,
	turnR,
	turn180,
	turnL,
	turnMax,	// 4
	/* REVIEW: this is a huge hack!!! needed for 180 when loop --
	   have to cross side line */
	turn180Special = (turn180 + turnMax),
};

/* for grfpath in TurnAtJct */
#define fpathL			0x01	// have good path to L
#define fpathC			0x02	// have good path in center (straight forward)
#define fpathR			0x04	// have good path to R
/* note: these are used only by caller... */
/* REVIEW: move to main.cpp?! */
#define fpathStop		0x80	// stop exploring; used with one of following...
#define fpathEnd		0x40	// found end circle!
#define fpathTime		0x20	// no more time
#define fpathUser		0x10	// user asked to stop

class MAZE
{
public:
	MAZE();

	/* record reaching junction (dist from prev location) and choose
	   a route (turn, possibly straight); returns the turn choice made */
	uchar TurnAtJct(ushort dist, uchar grfpath);

	/* save, load */
#ifdef WIN
	void CleanupAndSave();
#else
	void CleanupAndSave(const EEPROM &eeprom);	// saves (after appropriate cleanup)
	void Load(const EEPROM &eeprom);	// loads maze info (if there)
	void Clear(const EEPROM &eeprom);	// clears maze memory from eeprom
#endif

#ifdef OLD
#ifdef DEBUG
	void Show() const;
#endif
#endif

private:
	void Reset();
	void UpdateLocation(ushort dist);
	void ChangeDir(uchar turn);
	char FAddJct(uchar grfpath);
	uchar TurnExplore();
	uchar TurnRetrace();
	uchar TurnUnwind();
	uchar TurnFinishUnwind();
	uchar TurnCheckLoop();
	void FixSimpleJct(uchar *pgrfjct);

#ifdef OLD
	uchar TurnUnwind(uchar jctLoopStart);
	uchar Turn(uchar dirNew);
	uchar Dir(uchar turn) const;
	uchar DirOpp(uchar dir) const;
#ifdef DEBUG
	void ShowJct(uchar jct) const;
#endif
#endif

	/* REVIEW: new stuff... static?! */
#ifdef WIN
	uchar *m_pgrfjctMin;	// start of memory
#endif
	uchar *m_pgrfjctCur;	// current junction (last one saw for explore, retrace;
							//     one we're headed back to for rewind)
	uchar *m_pgrfjctMac;	// mac junction (used when retracing)
	uchar m_mss;			// maze solving state (see enum above)
	uchar m_dir;			// direction traveling
	short m_x, m_y;			// currrent location
};

#endif // MAZE_H
