/*	timer.h -- timer
*
*	Written by Cathy Saxton
*	cathy@idleloop.com
*/

#ifndef TIMER_H
#define TIMER_H

#ifdef CLK_1MHZ
/*
	clock = 1MHz (2^20)
	counter resets after 2^8 pulses (8-bit counter)
		-- have 2^12 to account for in overflows or prescaler
	prescaler at 64 (2^6) results in 2^6 (64) overflows per second
*/
#define ctickSec 64
#else
/*
	clock = 8MHz (2^23)
	counter resets after 2^8 pulses (8-bit counter)
		-- have 2^15 to account for in overflows or prescaler
	prescaler at 256 (2^8) results in 2^7 (128) overflows per second
*/
#define ctickSec 128
#endif

class TIMER
{
public:
	TIMER();

	void Wait(uchar sec) const;
	void WaitTicks(ushort ticks) const;

#ifdef UNUSED
	/* sets vars with elapsed time since last mark;
	   NULL OK for either/both param */
	void Mark(ushort *pctick, ushort *psec) const;
#endif

/* ushort: 0-64k; with 64 ticks/sec, can hold 1024 sec = 17 minutes */

	/* returns current tick count */
	ushort CtickCur() const;

	/* returns tick count for sec seconds from now */
	ushort CtickPlusSec(ushort sec) const;

private:
	void Init() const;

	static char m_fInit;
//	static ushort m_ctickMark;
};

#endif // TIMER_H
